﻿// Brick in the Wall by Milo Christiansen
//
// To the extent possible under law, the person who associated CC0 with
// this project has waived all copyright and related or neighboring rights
// to this project.
//
// You should have received a copy of the CC0 legalcode along with this
// work.  If not, see <http://creativecommons.org/publicdomain/zero/1.0/>.

using System.Text.RegularExpressions;

using Vintagestory.API;
using Vintagestory.API.Common;
using Vintagestory.API.MathTools;

namespace BrickTransitionBehavior {
	public class BrickTransitionBehaviorMod : ModSystem {
		public override void Start(ICoreAPI api) {
			base.Start(api);

			api.RegisterBlockBehaviorClass("BrickTransitionBehavior", typeof(BrickTransitionBehavior));
		}
	}

	class BrickTransitionBehavior : BlockBehavior {
		private Regex transitionsRe;

		public BrickTransitionBehavior(Block block) : base(block) {
			// NOP
		}

		public override bool TryPlaceBlock(IWorldAccessor world, IPlayer byPlayer, ItemStack itemstack, BlockSelection blockSel, ref EnumHandling handling, ref string failureCode) {
			handling = EnumHandling.PreventDefault;
			if (!world.BlockAccessor.GetBlock(blockSel.Position).IsReplacableBy(this.block)) {
				return false;
			}

			var code = new string[5];
			code[0] = "red";
			var m = transitionsRe.Match(this.block.Code.Path);
			if (m.Success) {
				code[0] = m.Result("${typ}");
			}
			code[1] = codeFor(world.BlockAccessor.GetBlock(blockSel.Position.NorthCopy()), code[0], "s");
			code[2] = codeFor(world.BlockAccessor.GetBlock(blockSel.Position.SouthCopy()), code[0], "n");
			code[3] = codeFor(world.BlockAccessor.GetBlock(blockSel.Position.EastCopy()), code[0], "w");
			code[4] = codeFor(world.BlockAccessor.GetBlock(blockSel.Position.WestCopy()), code[0], "e");

			world.BlockAccessor.SetBlock(world.BlockAccessor.GetBlock(block.CodeWithParts(code)).BlockId, blockSel.Position);
			return true;
		}

		private string codeFor(Block b, string def, string dir) {
			var m = transitionsRe.Match(b.Code.Path);
			if (m.Success) {
				return m.Result("${" + dir + "}");
			}
			return def;
		}

		public override ItemStack OnPickBlock(IWorldAccessor world, BlockPos pos, ref EnumHandling handling) {
			handling = EnumHandling.PreventDefault;
			var m = transitionsRe.Match(world.BlockAccessor.GetBlock(pos).Code.Path);
			if (m.Success) {
				var t = m.Result("${typ}");
				return new ItemStack(world.BlockAccessor.GetBlock(block.CodeWithParts(t, t, t, t, t)));
			}
			return new ItemStack(world.BlockAccessor.GetBlock(block.CodeWithParts("red", "red", "red", "red", "red")));
		}

		public override ItemStack[] GetDrops(IWorldAccessor world, BlockPos pos, IPlayer byPlayer, float dropQuantityMultiplier, ref EnumHandling handling) {
			handling = EnumHandling.PreventDefault;
			var m = transitionsRe.Match(world.BlockAccessor.GetBlock(pos).Code.Path);
			if (m.Success) {
				var t = m.Result("${typ}");
				return new ItemStack[1] { new ItemStack(world.BlockAccessor.GetBlock(block.CodeWithParts(t, t, t, t, t))) };
			}
			return new ItemStack[1] { new ItemStack(world.BlockAccessor.GetBlock(block.CodeWithParts("red", "red", "red", "red", "red"))) };
		}

		public override void Initialize(JsonObject properties) {
			base.Initialize(properties);
			var tr = "";
			var first = true;
			foreach (var mat in properties["transitions"].AsString("").Split(',')) {
				tr += (first ? "" : "|") + Regex.Escape(mat);
				first = false;
			}
			transitionsRe = new Regex(".+-(?<typ>" + tr + ")-(?<n>" + tr + ")-(?<s>" + tr + ")-(?<e>" + tr + ")-(?<w>" + tr + ")(?:-.+)?");
		}
	}
}
